@@ -180,11 +180,23 @@ enum {
 
 static guint signals[SIGNAL_LAST_SIGNAL];
 
+static void
+child_breakpoint_bin_notify_current_breakpoint_cb (AdwDialog *self)
+{
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CURRENT_BREAKPOINT]);
+}
+
 static gboolean
 map_tick_cb (AdwDialog *self)
 {
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
 
+  if (priv->force_closing) {
+    priv->tick_cb_id = 0;
+    priv->ticks = 0;
+    return G_SOURCE_REMOVE;
+  }
+
   priv->ticks++;
 
   /* If we're showing a bottom sheet, it has changed after the initial map,
@@ -306,11 +318,14 @@ set_focus (AdwDialog *self,
                                           focus_widget_notify_visible_cb, self);
     g_signal_handlers_disconnect_by_func (priv->focus_widget,
                                           focus_widget_notify_parent_cb, self);
+    g_clear_weak_pointer (&priv->focus_widget);
   }
 
   priv->focus_widget = focus;
 
   if (priv->focus_widget) {
+    g_object_add_weak_pointer (G_OBJECT (priv->focus_widget), (gpointer *) &priv->focus_widget);
+
     g_signal_connect_swapped (priv->focus_widget, "hide",
                               G_CALLBACK (unset_focus_widget), self);
     g_signal_connect_swapped (priv->focus_widget, "notify::visible",
@@ -574,18 +589,8 @@ window_close_request_cb (AdwDialog *self)
 {
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
 
-  if (priv->force_closing || adw_dialog_close (self)) {
-    GtkAccessibleRole role =
-      gtk_accessible_get_accessible_role (GTK_ACCESSIBLE (priv->window));
-
-    g_object_set (self, "accessible-role", role, NULL);
-
-    gtk_accessible_update_property (GTK_ACCESSIBLE (self),
-                                    GTK_ACCESSIBLE_PROPERTY_LABEL, priv->title,
-                                    -1);
-
+  if (priv->force_closing || adw_dialog_close (self))
     return GDK_EVENT_PROPAGATE;
-  }
 
   return GDK_EVENT_STOP;
 }
@@ -604,7 +609,6 @@ present_as_window (AdwDialog *self,
 {
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
   GtkWidget *titlebar;
-  GtkAccessibleRole role;
   GtkEventController *shortcut_controller;
   GtkShortcut *shortcut;
 
@@ -651,13 +655,6 @@ present_as_window (AdwDialog *self,
 
   g_signal_connect_swapped (priv->window, "close-request", G_CALLBACK (window_close_request_cb), self);
 
-  /* Transfer the accessible role onto the window */
-  role = gtk_accessible_get_accessible_role (GTK_ACCESSIBLE (self));
-  g_object_set (priv->window, "accessible-role", role, NULL);
-  g_object_set (self, "accessible-role", GTK_ACCESSIBLE_ROLE_GENERIC, NULL);
-
-  gtk_accessible_reset_property (GTK_ACCESSIBLE (self), GTK_ACCESSIBLE_PROPERTY_LABEL);
-
   gtk_window_present (GTK_WINDOW (priv->window));
 }
 
@@ -879,9 +876,11 @@ adw_dialog_dispose (GObject *object)
                                           focus_widget_notify_visible_cb, self);
     g_signal_handlers_disconnect_by_func (priv->focus_widget,
                                           focus_widget_notify_parent_cb, self);
-    priv->focus_widget = NULL;
+    g_clear_weak_pointer (&priv->focus_widget);
   }
 
+  g_clear_weak_pointer (&priv->last_focus);
+
   if (priv->default_widget) {
     g_signal_handlers_disconnect_by_func (priv->default_widget,
                                           unset_default_widget, self);
@@ -899,7 +898,8 @@ adw_dialog_dispose (GObject *object)
     priv->floating_sheet = NULL;
     priv->child_breakpoint_bin = NULL;
     priv->child = NULL;
-  } else if (priv->window) {
+  } else if (priv->child_breakpoint_bin &&
+             gtk_widget_get_parent (priv->child_breakpoint_bin) == GTK_WIDGET (self)) {
     /* It's an window-backed dialog */
     g_clear_pointer (&priv->child_breakpoint_bin, gtk_widget_unparent);
     priv->child = NULL;
@@ -1286,6 +1286,10 @@ adw_dialog_init (AdwDialog *self)
   g_object_bind_property (self, "height-request",
                           priv->child_breakpoint_bin, "height-request",
                           G_BINDING_DEFAULT);
+  g_signal_connect_swapped (priv->child_breakpoint_bin,
+                            "notify::current-breakpoint",
+                            G_CALLBACK (child_breakpoint_bin_notify_current_breakpoint_cb),
+                            self);
 }
 
 static void
@@ -1894,15 +1898,29 @@ adw_dialog_close (AdwDialog *self)
 
   priv = adw_dialog_get_instance_private (self);
 
+  if (!gtk_widget_get_parent (GTK_WIDGET (self))) {
+    g_critical ("Trying to close %s %p that's not presented", G_OBJECT_TYPE_NAME (self), self);
+    return FALSE;
+  }
+
   if (!priv->can_close) {
     g_signal_emit (self, signals[SIGNAL_CLOSE_ATTEMPT], 0);
     return FALSE;
   }
 
-  if (priv->window)
-    gtk_window_close (GTK_WINDOW (priv->window));
-  else
+  if (priv->window) {
+    GtkWidget *window = priv->window;
+    priv->window = NULL;
+
+    if (priv->closing_callback)
+      priv->closing_callback (self, priv->user_data);
+
+    g_signal_emit (self, signals[SIGNAL_CLOSED], 0);
+
+    gtk_window_close (GTK_WINDOW (window));
+  } else {
     adw_dialog_force_close (self);
+  }
 
   return TRUE;
 }
